<?php

namespace Drupal\commerce_heidelpay\PluginForm\Heidelpay;

use Drupal\commerce_payment\PluginForm\PaymentMethodAddForm as BasePaymentMethodAddForm;
use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Form\FormStateInterface;

/**
 * Heidelpay add-form class.
 */
class PaymentMethodAddForm extends BasePaymentMethodAddForm {

  /**
   * {@inheritdoc}
   */
  public function buildCreditCardForm(array $element, FormStateInterface $form_state) {
    $element['#attributes']['class'][] = 'heidelpayUI';
    $element['#attributes']['class'][] = 'form';

    // Set our key to settings array.
    /** @var \Drupal\commerce_heidelpay\Plugin\Commerce\PaymentGateway\HeidelpayOnSiteInterface $plugin */
    $plugin = $this->plugin;

    $element['#attached']['drupalSettings']['commerceHeidelpay'] = [
      'publicKey' => $plugin->getPublicKey(),
    ];

    // Populated by the JS library.
    $element['heidelpay_resource_id'] = [
      '#type' => 'hidden',
      '#attributes' => [
        'id' => 'heidelpay-resource-id',
      ],
    ];

    $element['card_number'] = [
      '#type' => 'item',
      '#required' => TRUE,
      '#validated' => TRUE,
      '#markup' => '<div id="card-element-id-number" class="heidelpayInput"></div>',
    ];

    $element['expiration'] = [
      '#type' => 'item',
      '#required' => TRUE,
      '#validated' => TRUE,
      '#markup' => '<div id="card-element-id-expiry" class="heidelpayInput"></div>',
    ];

    $element['security_code'] = [
      '#type' => 'item',
      '#required' => TRUE,
      '#validated' => TRUE,
      '#markup' => '<div id="card-element-id-cvc" class="heidelpayInput"></div>',
    ];

    $element['account_holder'] = [
      '#type' => 'item',
      '#required' => TRUE,
      '#validated' => TRUE,
      '#markup' => '<div id="card-element-id-holder" class="heidelpayInput"></div>',
      '#access' => $plugin->getConfiguration()['account_holder'] ?? FALSE,
    ];

    // To display validation errors.
    $element['payment_errors'] = [
      '#type' => 'markup',
      '#markup' => '<div id="error-holder"></div>',
      '#weight' => -200,
    ];

    $cacheability = new CacheableMetadata();
    $cacheability->addCacheableDependency($this->entity);
    $cacheability->setCacheMaxAge(0);
    $cacheability->applyTo($element);
    return $element;
  }

  /**
   * {@inheritdoc}
   */
  protected function validateCreditCardForm(array &$element, FormStateInterface $form_state) {
    // The JS library performs its own validation.
  }

  /**
   * {@inheritdoc}
   */
  public function submitCreditCardForm(array $element, FormStateInterface $form_state) {
    if ($email = $form_state->getValue(['contact_information', 'email'])) {
      $email_parents = array_merge($element['#parents'], ['email']);
      $form_state->setValue($email_parents, $email);
    }
  }

}
